<?php
/**
 * @file
 * Contains \Drupal\notify_log\Form\NotifyLogSettingsForm.
 */

namespace Drupal\notify_log\Form;

use Drupal\Component\Utility\Unicode as Unicode;
use \Drupal\Core\Form\ConfigFormBase as ConfigFormBase;
use \Drupal\Core\Form\FormStateInterface as FormStateInterface;
use \Drupal\Core\Logger\RfcLogLevel;

class NotifyLogSettingsForm extends ConfigFormBase {

  public function getFormId() {
    return 'notify_log_settings_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['notify_log_type'] = [
      '#type' => 'radios',
      '#title' => t('Notification way'),
      '#description' => t('Select the way of notification.'),
      '#options' => [
        NOTIFY_LOG_TYPE_JS => t('Browser (Notify.js)'),
        NOTIFY_LOG_TYPE_SYSTEM => t('System (Notify OSD)'),
      ],
      '#default_value' => $this->config('notify_log.settings')->get('notify_log_type'),
    ];
    if (!notify_log_system_capability()) {
      $form['notify_log_type'][NOTIFY_LOG_TYPE_SYSTEM] = ['#disabled' => TRUE];
      $form['notify_log_type']['#description'] .= '<br />' . t('"System" way available only for Linux based developers, also it needed some system configuration (see README.txt file).');
    }
    if (!notify_log_js_capability()) {
      $form['notify_log_type'][NOTIFY_LOG_TYPE_JS] = ['#disabled' => TRUE];

      // Update description.
      $description = t('Notify.js library is missed, please run <code>composer install</code> to solve it.');
      $form['notify_log_type']['#description'] .= '<br />' . $description;
    }

    $form['notify_log_severity'] = [
      '#type' => 'select',
      '#title' => t('Severity type'),
      '#description' => t('Select severity levels to display. If nothing chosen - display all types.'),
      '#options' => [
        RfcLogLevel::EMERGENCY => t('emergency'),
        RfcLogLevel::ALERT => t('alert'),
        RfcLogLevel::CRITICAL => t('critical'),
        RfcLogLevel::ERROR => t('error'),
        RfcLogLevel::WARNING => t('warning'),
        RfcLogLevel::NOTICE => t('notice'),
        RfcLogLevel::INFO => t('info'),
        RfcLogLevel::DEBUG => t('debug'),
      ],
      '#default_value' => $this->config('notify_log.settings')->get('notify_log_severity'),
      '#multiple' => TRUE,
    ];

    $form['notify_log_system_user'] = [
      '#type' => 'textfield',
      '#title' => t('System username'),
      '#description' => t('Enter username of your OS user.'),
      '#default_value' => $this->config('notify_log.settings')->get('notify_log_system_user'),
      '#states' => [
        'visible' => [
          ':input[name="notify_log_type"]' => ['value' => NOTIFY_LOG_TYPE_SYSTEM],
        ],
      ],
    ];

    $form['notify_log_js'] = [
      '#type' => 'details',
      '#title' => t('Notify.js settings'),
      '#open' => TRUE,
      '#states' => [
        'visible' => [
          ':input[name="notify_log_type"]' => ['value' => NOTIFY_LOG_TYPE_JS],
        ],
      ],
    ];

    $form['notify_log_js']['notify_log_js_click_to_hide'] = [
      '#type' => 'checkbox',
      '#title' => t('Click to hide'),
      '#description' => t('Whether to hide the notification on click.'),
      '#default_value' => $this->config('notify_log.settings')->get('notify_log_js_click_to_hide'),
    ];

    $types = [
      NOTIFY_LOG_TYPE_JS_INFO,
      NOTIFY_LOG_TYPE_JS_WARNING,
      NOTIFY_LOG_TYPE_JS_ERROR,
    ];

    foreach ($types as $type) {
      $fieldset_name = 'notify_log_js_' . $type;

      $form['notify_log_js'][$fieldset_name] = [
        '#type' => 'details',
        '#title' => t($type),
        '#open' => TRUE,
      ];

      $form['notify_log_js'][$fieldset_name][$fieldset_name  .'_auto_hide'] = [
        '#type' => 'checkbox',
        '#title' => t('Auto hide'),
        '#description' => t('Whether to auto-hide the notification.'),
        '#default_value' => $this->config('notify_log.settings')->get($fieldset_name  .'_auto_hide'),
      ];

      $form['notify_log_js'][$fieldset_name][$fieldset_name  .'_auto_hide_delay'] = [
        '#type' => 'textfield',
        '#title' => t('Auto hide delay'),
        '#description' => t('If auto-hide, hide after milliseconds.'),
        '#default_value' => $this->config('notify_log.settings')->get($fieldset_name  .'_auto_hide_delay'),
        '#states' => [
          'visible' => [
            ':input[name="' . $fieldset_name . '_auto_hide"]' => ['checked' => TRUE],
          ],
        ],
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->getValue('notify_log_type') == NOTIFY_LOG_TYPE_SYSTEM) {
      if (!$form_state->getValue('notify_log_system_user')) {
        $form_state->setErrorByName('notify_log_system_user', $this->t('You should specify system user name to use system notifications.'));
      }
      elseif (!notify_log_system_user_exists($form_state->getValue('notify_log_system_user'))) {
        $form_state->setErrorByName('notify_log_system_user', $this->t('There is no such user in your OS'));
      }
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('notify_log.settings')
      ->set('notify_log_type', $form_state->getValue('notify_log_type'))
      ->set('notify_log_severity', $form_state->getValue('notify_log_severity'))
      ->set('notify_log_system_user', $form_state->getValue('notify_log_system_user'))
      ->set('notify_log_js_click_to_hide', $form_state->getValue('notify_log_js_click_to_hide'))
      ->save();

    $types = [
      NOTIFY_LOG_TYPE_JS_INFO,
      NOTIFY_LOG_TYPE_JS_WARNING,
      NOTIFY_LOG_TYPE_JS_ERROR,
    ];
    $config = $this->config('notify_log.settings');
    foreach ($types as $type) {
      $fieldset_name = 'notify_log_js_' . $type;
      $config->set($fieldset_name . '_auto_hide', $form_state->getValue($fieldset_name . '_auto_hide'));
      $config->set($fieldset_name  .'_auto_hide_delay', $form_state->getValue($fieldset_name  .'_auto_hide_delay'));
    }
    $config->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Gets the configuration names that will be editable.
   *
   * @return array
   *   An array of configuration object names that are editable if called in
   *   conjunction with the trait's config() method.
   */
  public function getEditableConfigNames() {
    return ['notify_log.settings'];
  }
}
