<?php

namespace Drupal\notify_log\Logger;

use \Drupal\Component\Utility\Unicode;
use \Drupal\Core\Database\Connection;
use \Drupal\Core\Logger\LogMessageParserInterface;
use \Drupal\Core\Logger\RfcLoggerTrait;
use \Drupal\Core\Logger\RfcLogLevel;
use \Psr\Log\LoggerInterface;

class NotifyLogger implements LoggerInterface {
  use RfcLoggerTrait;

  public function __construct(Connection $connection, LogMessageParserInterface $parser) {
    $this->connection = $connection;
    $this->parser = $parser;
  }

  /**
   * {@inheritdoc}
   */
  public function log($level, $message, array $context = array()) {
    $access = TRUE;
    $config = \Drupal::config('notify_log.settings');

    if (!$notify_type = $config->get('notify_log_type')) {
      $access = FALSE;
    }

    // If it's already prepared object we should just render it.
    if ($message instanceof TranslatableMarkup) {
      $log_entry['message'] = $message->render();
    }
    // If it's not processed string, we should process it.
    else if (is_string($message)) {
      $message_placeholders = $this->parser->parseMessagePlaceholders($message, $context);
      $log_entry['message'] = t($message, $message_placeholders)->render();
    }
    // Unexpected situation.
    else {
      $log_entry['message'] = t('Notify log cannot handle this error message.');
    }

    $log_entry['severity'] = $level;
    $log_entry['type'] = Unicode::substr($context['channel'], 0, 64);

    if (!$access || !$this->notifyLogSeverity($log_entry['severity'])) {
      return;
    }

    if ($notify_type == NOTIFY_LOG_TYPE_SYSTEM) {
      if ($log_entry['severity'] <= RfcLogLevel::ERROR) {
        $style = NOTIFY_LOG_TYPE_SYSTEM_CRITICAL;
      }
      elseif ($log_entry['severity'] <= RfcLogLevel::NOTICE) {
        $style = NOTIFY_LOG_TYPE_SYSTEM_NORMAL;
      }
      else {
        $style = NOTIFY_LOG_TYPE_SYSTEM_LOW;
      }

      $user_name = escapeshellarg($config->get('notify_log_system_user'));
      $style = escapeshellarg($style);
      $type = escapeshellarg($log_entry['type']);
      $message = escapeshellarg($log_entry['message']);
      $cmd = 'export DISPLAY=:0; sudo -u ' . $user_name . ' notify-send -u ' . $style . ' "' . $type . '" "' . $message . '"';
      exec($cmd);
    }
    elseif ($notify_type == NOTIFY_LOG_TYPE_JS) {
      $entries = &drupal_static('notify_log_js_errors', []);
      $entries[] = $log_entry;
    }
  }

  /**
   * Check $severity type for capability.
   *
   * @param int $severity
   *   Severity level, @see watchdog_severity_levels() for more information.
   *
   * @return bool
   *   Indicator of available of using this $severity type
   */
  protected function notifyLogSeverity($severity) {
    $severities = \Drupal::config('notify_log.settings')->get('notify_log_severity');

    if (!$severities) {
      return TRUE;
    }

    return in_array($severity, $severities);
  }
}
